﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Devices;
using Microsoft.Xna.Framework.Media;

namespace Phone相机7
{
    public partial class MainPage : PhoneApplicationPage
    {
        // 构造函数
        public MainPage()
        {
            InitializeComponent();
        }

        PhotoCamera photoCamera;
        MediaLibrary library = new MediaLibrary();

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            //判断是否支持相机
            if (PhotoCamera.IsCameraTypeSupported(CameraType.Primary))
            {
                textblock1.Text = "当前取景：后置摄像头";
                photoCamera = new PhotoCamera(CameraType.Primary);
            }
            else if (PhotoCamera.IsCameraTypeSupported(CameraType.FrontFacing))
            {
                textblock1.Text = "当前取景：前置摄像头";
                photoCamera = new PhotoCamera(CameraType.FrontFacing);
            }
            else
            {
                textblock1.Text = "设备不支持相机";
                return;
            }
            textblock2.Text = "相机正在初始化中.";
            viewfinderBrush.SetSource(photoCamera);
            //相机初始化完成时
            photoCamera.Initialized += (a, b) =>
            {
                if (b.Succeeded)
                {
                    this.Dispatcher.BeginInvoke(() =>
                    {
                        SupportedFlashModesInit();
                        AvailableResolutionsInit();
                        textblock2.Text = "相机初始化完成.";
                    });
                }
            };

            //有图像可用时（拍摄完成）
            photoCamera.CaptureImageAvailable += (a, b) =>
            {
                this.Dispatcher.BeginInvoke(() => { textblock2.Text = "正在保存照片."; });
                library.SavePictureToCameraRoll(Guid.NewGuid().ToString() + ".jpg", b.ImageStream);
                this.Dispatcher.BeginInvoke(() => { textblock2.Text = "照片保存成功."; });
            };

            //对焦完成时
            photoCamera.AutoFocusCompleted += (a, b) =>
            {
                this.Dispatcher.BeginInvoke(() =>
                {
                    textblock2.Text = "自动对焦完成.";
                    focusBrackets.Visibility = Visibility.Collapsed;
                });
            };

            base.OnNavigatedTo(e);
        }

        /// <summary>
        /// 显示支持的闪光模式
        /// </summary>
        private void SupportedFlashModesInit()
        {
            List<string> flashModes = new List<string>() { "关" };
            if (photoCamera.IsFlashModeSupported(FlashMode.On)) flashModes.Add("开");
            if (photoCamera.IsFlashModeSupported(FlashMode.Auto)) flashModes.Add("自动");
            if (photoCamera.IsFlashModeSupported(FlashMode.RedEyeReduction)) flashModes.Add("红眼");
            listbox1.ItemsSource = flashModes;
        }

        /// <summary>
        /// 显示支持的分辨率
        /// </summary>
        private void AvailableResolutionsInit()
        {
            listbox2.ItemsSource = photoCamera.AvailableResolutions;
        }

        //对焦
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            //是否支持自动对焦
            if (photoCamera.IsFocusSupported == true)
            {
                try
                {
                    photoCamera.Focus();
                }
                catch(Exception ex)
                {
                    this.Dispatcher.BeginInvoke(() => { textblock2.Text = "对焦错误：" + ex.Message; });
                }
            }
            else
            {
                this.Dispatcher.BeginInvoke(() => { textblock2.Text = "相机不支持自动对焦."; });
            }
        }

        //拍照
        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            if (photoCamera != null)
            {
                try
                {
                    photoCamera.CaptureImage();
                }
                catch (Exception ex)
                {
                    this.Dispatcher.BeginInvoke(() => { textblock2.Text = "拍照错误：" + ex.Message; });
                }
            }
        }

        //特定点对焦
        private void canvas1_Tap(object sender, GestureEventArgs e)
        {
            if (photoCamera == null) return;
            if (!photoCamera.IsFocusAtPointSupported)
            {

                textblock2.Text = "不支持特定点对焦";
                return;
            }

            try
            {
                Point tapLocation = e.GetPosition(canvas1);

                focusBrackets.SetValue(Canvas.LeftProperty, tapLocation.X - 30);
                focusBrackets.SetValue(Canvas.TopProperty, tapLocation.Y - 28);

                double focusXPercentage = tapLocation.X / canvas1.Width;
                double focusYPercentage = tapLocation.Y / canvas1.Height;

                focusBrackets.Visibility = Visibility.Visible;
                photoCamera.FocusAtPoint(focusXPercentage, focusYPercentage);

                this.Dispatcher.BeginInvoke(delegate()
                {
                    textblock2.Text = String.Format("针对位置 [{0:N2} , {1:N2}] 开始对焦", focusXPercentage, focusYPercentage);
                });
            }
            catch (Exception focusError)
            {

                this.Dispatcher.BeginInvoke(delegate()
                {
                    textblock2.Text ="对焦错误："+ focusError.Message;
                    focusBrackets.Visibility = Visibility.Collapsed;
                });
            }

        }

        //切换闪光灯模式
        private void listbox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0) return;
            switch (e.AddedItems[0].ToString())
            {
                case "关":
                    photoCamera.FlashMode = FlashMode.Off;
                    break;
                case "自动":
                    photoCamera.FlashMode = FlashMode.Auto;
                    break;
                case "红眼":
                    photoCamera.FlashMode = FlashMode.RedEyeReduction;
                    break;
                case "开":
                    photoCamera.FlashMode = FlashMode.On;
                    break;
            }
            textblock2.Text = "已将闪光模式设置为：" + e.AddedItems[0].ToString();
        }
        //切换分辨率
        private void listbox2_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0) return;
            photoCamera.Resolution = (Size)e.AddedItems[0];
            textblock2.Text = "已将分辨率设置为：" + photoCamera.Resolution.ToString();
        }

    }
}